"""
bp_client ping module.

Basic package to test end to end communication
"""


import requests
import json

import bpclient


#####################################################

def Ping(access_key,data):
	"""Echo to BP Broker host adn recieve back echoed response.

	With payload data=test, receives json object:
	{"src": "127.0.0.1", "pong": {"data": "test"}}

	:param data: Data to be sent/echoed
	:returns pong: Original data in returned
	"""
	r = requests.post("https://%s/ping/Ping/" % bpclient.BPBROKER,params={'data': data, 'access_key': access_key},verify=False)
	return(r.json())





"""
bp_broker API module.

Listens and responds to ssl connections.  Proxies connection requests to registered event handlers.
"""


import os
import subprocess
from distutils import dir_util, file_util

import bpbroker


#####################################################

INIT_D_HEADER_RPM = """#!/bin/bash

# bpbroker daemon
# chkconfig: 345 25 85
# description: bpbroker service
# processname: bpbroker

"""

INIT_D_HEADER_DEB = """#!/bin/bash

### BEGIN INIT INFO
# Provides:          bpbroker
# Required-Start:    $syslog $remote_fs
# Required-Stop:     
# Default-Start:     3 4 5
# Default-Stop:      0 1 6
# Short-Description: bpbroker service
# Description:       BP Broker service
### END INIT INFO

"""

INIT_D_SCRIPT = """
DAEMON_PATH="/usr/local/bpbroker/bin/"

DAEMON=bpbroker
DAEMONOPTS="start"

NAME=bpbroker
DESC="Blueprint Broker Service"
PIDFILE=/var/run/$NAME.pid
SCRIPTNAME=/etc/init.d/$NAME

case "$1" in
start)
	printf "%-50s" "Starting $NAME..."
	cd $DAEMON_PATH
	PID=`./$DAEMON $DAEMONOPTS > /dev/null 2>&1 & echo $!`
	#echo "Saving PID" $PID " to " $PIDFILE
        if [ -z $PID ]; then
            printf "%s\n" "Fail"
        else
            echo $PID > $PIDFILE
            printf "%s\n" "Ok"
        fi
;;
status)
        printf "%-50s" "Checking $NAME..."
        if [ -f $PIDFILE ]; then
            PID=`cat $PIDFILE`
            if [ -z "`ps axf | grep ${PID} | grep -v grep`" ]; then
                printf "%s\n" "Process dead but pidfile exists"
            else
                echo "Running"
            fi
        else
            printf "%s\n" "Service not running"
        fi
;;
stop)
        printf "%-50s" "Stopping $NAME"
            PID=`cat $PIDFILE`
            cd $DAEMON_PATH
        if [ -f $PIDFILE ]; then
            kill -HUP $PID
            printf "%s\n" "Ok"
            rm -f $PIDFILE
        else
            printf "%s\n" "pidfile not found"
        fi
;;

restart)
  	$0 stop
  	$0 start
;;

*)
        echo "Usage: $0 {status|start|stop|restart}"
        exit 1
esac
"""

#####################################################




def _InstallLinux():
	global INIT_D_SCRIPT

	## Write init file ##
	if os.path.exists("/sbin/chkconfig"):
		with open("/etc/init.d/bpbroker","w") as f:  f.write(INIT_D_HEADER_RPM)
	elif os.path.exists("/usr/sbin/update-rc.d"):
		with open("/etc/init.d/bpbroker","w") as f:  f.write(INIT_D_HEADER_DEB)
	else:
		raise(Exception("Unable to install service, not RPM or DEB system"))
	with open("/etc/init.d/bpbroker","aw") as f:  f.write(INIT_D_SCRIPT)
	os.system("chmod oug+x /etc/init.d/bpbroker")
	
	## Dump current configuration ##
	if not os.path.exists("/usr/local/bpbroker/etc"):  os.makedirs("/usr/local/bpbroker/etc")
	if not os.path.exists("/usr/local/bpbroker/lib"):  os.makedirs("/usr/local/bpbroker/lib")
	bpbroker.config.Save("/usr/local/bpbroker/etc/bpbroker.json")

	## Install and start ##
	error = False
	if os.path.exists("/sbin/chkconfig"):
		# RHEL or RPM based
		error = os.system("/sbin/chkconfig --add bpbroker && /sbin/chkconfig bpbroker on && /sbin/service bpbroker start")
	elif os.path.exists("/usr/sbin/update-rc.d"):
		error = os.system("/usr/sbin/update-rc.d bpbroker enable 3 4 5 && /usr/sbin/service bpbroker start")
	else:
		raise(Exception("Unable to install service, not RPM or DEB system"))

	if error:  raise(Exception("OS error %s executing service install" % error))


def _InstallWindows():
	## Dump current configuration ##
	if not os.path.exists("%s/bpbroker/etc" % os.environ["ProgramW6432"]):  os.makedirs("%s/bpbroker/etc" % os.environ["ProgramW6432"])
	if not os.path.exists("%s/bpbroker/lib" % os.environ["ProgramW6432"]):  os.makedirs("%s/bpbroker/lib" % os.environ["ProgramW6432"])
	if not os.path.exists("%s/bpbroker/bin" % os.environ["ProgramW6432"]):  os.makedirs("%s/bpbroker/bin" % os.environ["ProgramW6432"])
	bpbroker.config.Save("%s/bpbroker/etc/bpbroker.json" % os.environ["ProgramW6432"])

	# Install service
	if os.path.exists("%s\\bpbroker\\bin\\nssm.exe" % os.environ["ProgramW6432"]):
		subprocess.call(["%s\\bpbroker\\bin\\nssm.exe" % os.environ["ProgramW6432"], "stop", "bpbroker"])
		subprocess.call(["%s\\bpbroker\\bin\\nssm.exe" % os.environ["ProgramW6432"], "remove", "bpbroker", "confirm"])
		error = subprocess.call(["%s\\bpbroker\\bin\\nssm.exe" % os.environ["ProgramW6432"],
		                         "install",
								 "bpbroker",
								 "%s\\bpbroker\\Python27\\Scripts\\bpbroker.exe" % os.environ["ProgramW6432"],
								 "start"])
		if error:  raise(Exception("OS error %s installing service: " % error))
	else:
		raise(Exception("nssm.exe not present as expected at %s\\bpbroker\\nssm.exe" % os.environ["ProgramW6432"]))

	# Start service
	error = subprocess.call(["%s\\bpbroker\\bin\\nssm.exe" % os.environ["ProgramW6432"],
	                         "start",
							 "bpbroker"])
	if error:  raise(Exception("OS error %s executing service start following install" % error))


def _UninstallLinux():
	pass


def _UninstallWindows():
	subprocess.call(["%s\\bpbroker\\bin\\nssm.exe" % os.environ["ProgramW6432"], "stop", "bpbroker"])
	subprocess.call(["%s\\bpbroker\\bin\\nssm.exe" % os.environ["ProgramW6432"], "remove", "bpbroker", "confirm"])


def Install():
	if os.name=='nt':  _InstallWindows()
	else:  _InstallLinux()


def Uninstall():
	if os.name=='nt':  _UninstallWindows()
	else:  _UninstallLinux()


def InstallExtension(script):
	"""Copy a python script / package directory to the bp broker system lib.  Used to easily extend bpbroker functionality."""

	if os.name == "nt":  bpbroker_dir = "%s/bpbroker" % os.environ["ProgramW6432"]
	elif os.name == "posix":  bpbroker_dir = "/usr/local/bpbroker"

	try:
		if os.path.isfile(script):  file_util.copy_file(script,bpbroker_dir+"/lib/")
		else:  dir_util.copy_tree(script,bpbroker_dir+"/lib/")
	except Exception as e:
		raise(Exception("Error installing extension %s: %s" % (script,str(e))))






"""
Access and modify saved/running config.

This is thread safe.

"""


import os
import sys
import json
import threading

import bpbroker

default_config = {
	'api': {
	},
	'worker':  {
	},
	'_global':  {
		'healthcheck_freq_sec': 10,
	},
	'_config':  {
		'backup_freq_secs': 3600,
		'backup_retain_n': 24,
	},

	'ping':  { },
	'services':  { },
}


#####################################################


def ImportConfigString(cstr):
	try:
		with bpbroker.config.rlock:
			bpbroker.config.data =  dict(list(bpbroker.config.data.items()) + list(json.loads(cstr).items()))
			bpbroker.config.Save()
	except Exception as e:
		raise(Exception("Unable to import configuration: %s" % str(e)))


def ImportConfigFile(cfile):
	try:
		with open(cfile) as f:  
			ImportConfigString(f.read())
	except Exception as e:
		raise(Exception("Unable to import config file %s: %s\n" % (cfile,str(e))))


class Config(object):

	def __init__(self, source=False):
		self.data = {}
		self.source = source
		self.rlock = threading.RLock()

		if not self.source and os.name == 'posix':
			self.source = "/usr/local/bpbroker/etc/bpbroker.json"
		elif not self.source and os.name == 'nt':
			self.source = "%s/bpbroker/etc/bpbroker.json" % os.environ["ProgramW6432"]
		self.data = default_config
		if self.source:  self._Load()


	def __exit__(self, type, value, traceback):
		if self.source:  self.Save()


	def _Load(self):
		with self.rlock:
			try:
				with open(self.source) as fp:
					self.data =  dict(list(self.data.items()) + list(json.load(fp).items()))
			except IOError:
				# File doesn't exist - set note and save to self.source on program exit
				sys.stderr.write("No configuration file at %s, running with default configuration\n" % str(self.source))
			except:
				raise


	def Save(self,source=False):
		if source:  self.source = source

		if self.source:  
			with self.rlock:
				try:
					with open(self.source,"w") as fp:
						self.data = json.dump(self.data,fp)
				except Exception as e:
					raise(Exception("Error saving configuration file %s: %s" % (self.source,str(e))))







# -*- coding: utf-8 -*-
"""
CenturyLink Cloud Blueprint Broker Toolset.

This broker foundation works to support advanced Blueprint deployment requiring
shared state and multi-server/multi-role communication across multiple server deployments.

CenturyLink Cloud: http://www.CenturyLinkCloud.com
Package Github page: 

"""

import sys
import os

## Add local path to support imported custom extensions
if os.name == 'posix':  sys.path.append("/usr/local/bpbroker/lib")
elif os.name == 'nt':  sys.path.append("%s/bpbroker/lib" % os.environ["ProgramW6432"])

from bpbroker.shell import Args, ExecCommand
import bpbroker.cli as cli
import bpbroker.server as server
import bpbroker.api as API
import bpbroker.discover as discover
import bpbroker.ping as ping
import bpbroker.services as services
import bpbroker.config as config_class
import bpbroker.install as install

#if os.name == 'nt':  import bpbroker.windows_service as windows_service

####### module/object vars #######
#_V1_API_KEY = False

config = config_class.Config()




"""Command-line interface to bpbroker."""

import argparse
import ConfigParser
import os
import sys
import bpbroker


class Args:

	def __init__(self):
		bpbroker.args = self
		self.ParseArgs()
		#self.ImportIni()
		self.MergeEnvironment()
		self.MergeCommands()


	def ParseArgs(self):
		parser = argparse.ArgumentParser(description="bpbroker service")
		parser_sp1 = parser.add_subparsers(title='Commands',dest='command')

		########## Run Service ###########
		parser_install = parser_sp1.add_parser('start', help='Start service')


		########## Install Service ###########
		parser_sp1.add_parser('install-service', help='Install and start service')
		parser_sp1.add_parser('uninstall-service', help='Remove service')


		########## Install Extension ###########
		parser_extension = parser_sp1.add_parser('install-extension', help='Install custom extension')
		parser_extension.add_argument('--script', required=True, help='Python script or package directory to install')


		########## Configure ###########
		parser_configure = parser_sp1.add_parser('configure', help='Apply additional configuration to bpbroker service')
		parser_configure.add_argument('--config-file', help='File containing configuration json to apply.  Default is stdin')


		########## Global ###########
		parser.add_argument('--config', '-c', help='Path to non-default configuration file')
		self.args = parser.parse_args()


	def GetCommand(self):  return(self.args.command)
	def GetArgs(self):  return(self.args)


	def ImportIni(self):
		pass


	def MergeEnvironment(self):
		#if 'BPBROKER' in os.environ:  bpbroker.BPROKER = os.environ['BPROKER']
		pass


	def MergeCommands(self):
		#if self.args.bpbroker:  bpbroker.BPBROKER = self.args.bpbroker
		pass




class ExecCommand():
	def __init__(self):
		try:
			if bpbroker.args.args.config:  bpbroker.config = bpbroker.config_class.Config(bpbroker.args.args.config)
			self.Bootstrap()
		except Exception as e:
			sys.stderr.write("Fatal error: %s\n" % str(e))
			sys.exit(1)


	def Bootstrap(self):
		if bpbroker.args.GetCommand() == 'start':  self.Start()
		elif bpbroker.args.GetCommand() == 'install-service':  self.Install()
		elif bpbroker.args.GetCommand() == 'uninstall-service':  self.Uninstall()
		elif bpbroker.args.GetCommand() == 'configure':  self.Configure()
		elif bpbroker.args.GetCommand() == 'install-extension':  self.InstallExtension()


	def Start(self):
		bpbroker.server.Start()


	def Install(self):
		bpbroker.install.Install()


	def Uninstall(self):
		bpbroker.install.Uninstall()


	def InstallExtension(self):
		bpbroker.install.InstallExtension(bpbroker.args.args.script)


	def Configure(self):
		if bpbroker.args.args.config_file:  bpbroker.config_class.ImportConfigFile(bpbroker.args.args.config_file)
		else:  bpbroker.config_class.ImportConfigString("tbd")


